// $License: NOLICENSE 

/**@file
 *
 * This library contains all functions that serve as interface between the NetMonitor panel and database.
 *
 * @author Wojciech Jan Fabian (EN/ICE)
 * @version 1.0.0
 * @date   November 2013
 */

#uses "CtrlRDBAccess"

//CtrlRDBAccess DB Connection timeout:
const int FW_NET_MONITOR_CONNECTION_TIMEOUT = 600;

// Value returned in case of error
const int FW_NET_MONITOR_ERROR_RETURN = -1;

// Number of outdated version of WinCC OA
const string FW_NET_MONITOR_OLD_VERSION = "3.6";

// DP Types of devices
const string FW_NET_MONITOR_DPT_PLC = "IcemonPlc";
const string FW_NET_MONITOR_DP_PLC_PREFIX = "plc-*.:";
const string FW_NET_MONITOR_DPT_PC = "IcemonHost";
const string FW_NET_MONITOR_DP_PC_PREFIX = "host-*.:";
const string FW_NET_MONITOR_SWITCH_DP_PREFIX = "fwNetMonitorSwitch/";
const string FW_NET_MONITOR_SWITCH_DP_TYPE = "FwNetMonitorSwitch";

//DB Connection using RDBAccess.ctl
global dbConnection gFwNetMonitorConn;
string FW_NET_MONITOR_CONNECTION_NAME = "fwNetMonitorConnection"; // can't be const to work smoothly

// TODO Would it be interesting to have this somewhere globally? Answer: Yes, Stefan - it would be interesting.
/** This function opens a connection to the System Configuration DB
  @param database (in) database server name
  @param username (in) user name 
  @param password (in) password
  @param owner (in) schema owner
  @param driver (in) driver to be used. By default "QOCI8".
  @return 0 if OK, -1 if error
*/
synchronized int fwNetMonitor_openDBConnection(string database, 
                                                   string username, 
                                                   string password, 
                                                   string owner, 
                                                   string driver = "QOCI8")
{
  //DebugTN("Opening connection.");
  string errTxt;
  string connectionString = "driver="+driver+";database="+database+";uid="+username+";enc_pwd=" + password;
  string sql;
  
  //-----------------------------------
  // Setting the parameters for our future connection
  //-----------------------------------
  
  if(fwNetMonitor_setConnectionTimeout() < 0)
  {
    DebugTN("fwNetMonitor_openDBConnection()->Error setting DB connection timeout");
    return FW_NET_MONITOR_ERROR_RETURN;
  }
  
  int returnCode = rdbGetConnection(FW_NET_MONITOR_CONNECTION_NAME, gFwNetMonitorConn);
  if (returnCode == FW_NET_MONITOR_ERROR_RETURN){
        rdbCheckError(errTxt, gFwNetMonitorConn);
        DebugTN("fwNetMonitor_openDBConnection()->Error getting the DB connection: " + errTxt);
        return FW_NET_MONITOR_ERROR_RETURN;
  }
  else if(returnCode == 0) //i.e. we alredy have a valid connection.
  {
    return 0;
  }
  
  //-----------------------------------
  // Open the actual connection  
  //-----------------------------------  
  
  // means: no connection yet... open on
  DebugTN("Trying to establish DB connection. DB server:  " +  database + ", Username : " + username + ", Schema Owner account: " + owner, "INFO");
  int returnCode = rdbOpenConnection(connectionString, gFwNetMonitorConn, FW_NET_MONITOR_CONNECTION_NAME);
  if (rdbCheckError(errTxt,gFwNetMonitorConn)){
    DebugTN("fwNetMonitor_openDBConnection() -> ERROR WHILE OPENING DATABASE CONNECTION: " + errTxt, "ERROR");
    return FW_NET_MONITOR_ERROR_RETURN;
  };      
  
  //-----------------------------------
  // SQL command that sets current_schema to schema owner 
  //-----------------------------------
  
  if((owner != "") && (owner != username))
  {  
    sql = "ALTER SESSION SET CURRENT_SCHEMA = " + owner;
    DebugTN("INFO: DB Connection Established. Setting current DB schema to " +  owner, "INFO");
    rdbExecuteSqlStatement (gFwNetMonitorConn, sql);
    if (rdbCheckError(errTxt, gFwNetMonitorConn))
    {
      fwNetMonitor_closeDBConnection();//force close connection if fails to set current schema
      DebugTN("fwNetMonitor_openDBConnection() -> DB ERROR setting current schema to schema owner: " + errTxt, "ERROR", 7);
      return FW_NET_MONITOR_ERROR_RETURN;
    }

    //Make sure that the current schema is set to schema owner after reconnection:
    rdbSetSqlOnReconnect(gFwNetMonitorConn, sql);   
  
    if (rdbCheckError(errTxt, gFwNetMonitorConn))
    {
      fwNetMonitor_closeDBConnection();//force close connection if fails to set current schema
      DebugTN("fwNetMonitor_openDBConnection() -> Error establishing schema owner callback" + errTxt, "ERROR");
      return FW_NET_MONITOR_ERROR_RETURN;
    }
  }
  
  return 0;
}

/** This function closes the connection to the System Configuration DB, keeping the global variable separated
  @return 0 if OK, -1 if error
*/
int fwNetMonitor_closeDBConnection()
{
  //DebugTN("Closing connection.");
  return rdbCloseConnection(gFwNetMonitorConn); 
}

/** This function executes a SQL query
  @param sql (in) sql sentence to be executed
  @param data (out) result of the query as a dyn_dyn_mixed
  @return 0 if OK, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_executeQuery(string sql, dyn_dyn_mixed &data)
{
  dbCommand cmd;
  string errTxt;
  rdbStartCommand(FW_NET_MONITOR_CONNECTION_NAME, sql, cmd);
  
  if (rdbCheckError(errTxt, gFwNetMonitorConn))
  {
    DebugTN("fwNetMonitor_executeQuery() -> DB Error: "+errTxt + + sql);
    // rdbFinishCommand(cmd); 
    return FW_NET_MONITOR_ERROR_RETURN;
  };

  
  dyn_mixed record; // not sure why, but is needed for rdbBindParams(cmd, record);
  rdbBindParams(cmd, record);
  if(fwNetMonitor_checkDBerror(sql, cmd, errTxt) != 0)
  {
    return FW_NET_MONITOR_ERROR_RETURN;
  }

  rdbExecuteCommand(cmd);
  if(fwNetMonitor_checkDBerror(sql, cmd, errTxt) != 0)
  {
    return FW_NET_MONITOR_ERROR_RETURN;
  }
   
  rdbGetData(cmd, data);
  if(fwNetMonitor_checkDBerror(sql, cmd, errTxt) != 0)
  {
    return FW_NET_MONITOR_ERROR_RETURN;
  }
  
  rdbFinishCommand(cmd);
  // note that we do not put the second parameter here!
  if (rdbCheckError(errTxt))
  {
    DebugTN("fwNetMonitor_executeQuery() -> DB Error: "+ errTxt + " " + sql);
    return FW_NET_MONITOR_ERROR_RETURN;
  };
  
  return 0;
}

/** This function checks a database error
  @param sql (in) sql sentence to be executed
  @param cmd (out) executed command
  @param errTxt (out) error text
  @return 0 if OK, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_checkDBerror(string sql, dbCommand cmd, string errTxt)
{
  if (rdbCheckError(errTxt, cmd))
  {
    DebugTN("fwNetMonitor_executeQuery() -> DB ERROR" +errTxt+" "+sql); 
    rdbFinishCommand(cmd); 
    return FW_NET_MONITOR_ERROR_RETURN;
  };
  return 0;
}

/** This function sets the status of the DB connection initialization
  @param value Initialization status
  @return 0 if OK, -1 if error
*/
int fwNetMonitor_setInitialized(bool value)
{
  return dpSetWait("fwNetMonitor_database.initialized", value);
}

/** This function sets the flag for the DB-agent of the installation tool to use System Configuration DB
  @param useDB flag indicating whether the DB must be accessed or not
  @return 0 if OK, FW_NET_MONITOR_ERROR_RETURN if errors
*/
int fwNetMonitor_setUseDB(bool useDB)
{
  return dpSetWait("fwNetMonitor_database.useDB", useDB);
}

/** This function sets the idle DB connection timeout to the default value
  @return 0 if OK, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_setConnectionTimeout()
{
  mixed connectionData;
  int connectionTimeout = FW_NET_MONITOR_CONNECTION_TIMEOUT;
  int timeOut = 0;

  if(VERSION != FW_NET_MONITOR_OLD_VERSION)
  {
    if(rdbOption("GetConnectionTimeout", 0, connectionData))
    {
      DebugTN("fwNetMonitor_throw_setConnectionTimeout()->Error retriving the current value of the DB connection timeout");
      return FW_NET_MONITOR_ERROR_RETURN;
    }
    timeOut = connectionData;
  }
  
  
  if(timeOut == FW_NET_MONITOR_CONNECTION_TIMEOUT)
  {
    return 0; //Nothing to be done as the value of the timeout is the desired one
  } 
  
  if(rdbOption("SetConnectionTimeout", connectionTimeout, connectionData))
  {
    DebugTN("fwNetMonitor_throw_setConnectionTimeout()->Failed to set DB connection timeout to " + FW_NET_MONITOR_CONNECTION_TIMEOUT);
    return FW_NET_MONITOR_ERROR_RETURN;
  }
  return 0;
}

/** This function gets all devices and their switches from DB
  @return devices and switches, FW_NET_MONITOR_ERROR_RETURN if error
*/
dyn_dyn_mixed fwNetMonitor_getSwitchesData()
{
  dyn_dyn_mixed queryResult;
  string server, username, password, owner;
  
  if(dpExists("fwNetMonitor_database"))
  {
    dpGet("fwNetMonitor_database.server", server,
         "fwNetMonitor_database.username", username,
         "fwNetMonitor_database.password", password,
         "fwNetMonitor_database.schemaOwner", owner);
    
    if(fwNetMonitor_openDBConnection(server, username, password, owner) != 0)
    {
        ChildPanelOnCentral("vision/MessageInfo1","Connection Failed", 
  	                   makeDynString("$1: Database connection failed.\nCheck connection parameters"));
    }
    else
    {  
      string sql;

      dynClear(queryResult);
      //dyn_mixed var; // var is needed for rdbBindParams(cmd, record);
    
      sql = "select unique usual_name, switch_name from landb.connections"; // this magic query is only used here
    
      //fwNetMonitor_executeQuery(sql, var, queryResult);
      fwNetMonitor_executeQuery(sql, queryResult);
    }
  
    //Force closing connection in any case.  
    fwNetMonitor_closeDBConnection();
    return queryResult;
  }
  else
  {
    DebugTN("fwNetMonitor_ throw: DP fwNetMonitor_database does not exist!");
    return FW_NET_MONITOR_ERROR_RETURN;
  }
}

/** This function gets all hosts and PLCs under MOON
  @return hosts and PLCs
*/
dyn_string fwNetMonitor_getAllDevices()
{
  dyn_string allPCs = fwNetMonitor_getDevicesByType(FW_NET_MONITOR_DP_PC_PREFIX, FW_NET_MONITOR_DPT_PC);
  dyn_string allPLCs = fwNetMonitor_getDevicesByType(FW_NET_MONITOR_DP_PLC_PREFIX, FW_NET_MONITOR_DPT_PLC);
  dynAppend(allPCs, allPLCs);

  return allPCs;
} 

/** This function gets devices of declared type
  @param searchPrefix (in) search template, e.g. "host-*.:"
  @param dpType (in) the datapoint type of desired devices
  @return devicesNames
*/
dyn_string fwNetMonitor_getDevicesByType(string searchPrefix, string dpType)
{
  dyn_string devicesNames = dpGetAlias(dpNames(searchPrefix, dpType));
  return devicesNames;
} 

/** This function creates new switch DP
  @param switchName (in) name of the switch (not DP address)
  @return 0
*/
int fwNetMonitor_createSwitch(string switchName)
{
  string switchDp = fwNetMonitor_getSwitchDp(switchName);
  dpCreate(switchDp, FW_NET_MONITOR_SWITCH_DP_TYPE);
  //DebugTN("Created switch: " + switchDp);
  dpSetWait(switchDp + ".name", strtoupper(switchName));
  fwNetMonitor_setMultiInstanceAlarmConfig(switchDp + ".alarm");
  return 0;  
}

/** This function set multi instance alert configuration for the given dpe
  @param dpe (in) datapoint element to be configured (int or bool type)
  @return 0
*/
int fwNetMonitor_setMultiInstanceAlarmConfig(string dpe)
{
  //DebugTN("Setting alert config...");
  dyn_mixed alarmObject;
  dyn_string exceptionInfo;
  //create the alarm object
  fwAlertConfig_objectCreateMultiInstance( 
                  alarmObject, //the object that will contain the alarm settings
                  makeDynString( "Ok came", "not Ok came"), // State info
                  makeDynString( "*","2"), // Random thing for mulitiinstance alerts
                  makeDynString("","_fwErrorNack_70."), // Alert type
                  "", //alarm panel, if necessary
                  makeDynString(""), //$-params to pass to the alarm panel, if necessary
                  "", //alarm help, if needed
                  false, //impulse alarm
                  makeDynBool(0,0), //negation of the matching (0 means "=", 1 means "!=")
                  "", //state bits that must also match for the alarm
                  makeDynString("",""), //state bits that must match for each range
                  exceptionInfo ); //exception info returned here
        
  //set the alarm to the dpe     
  fwAlertConfig_objectSet(dpe, alarmObject, exceptionInfo);   
  //activate it
 
  fwAlertConfig_activate(dpe, exceptionInfo); 
  
  return 0;
}

/** This function removes switch DP
  @param switchName (in) name of the switch (not DP address)
  @return 0 if OK, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_deleteSwitch(string switchName)
{
  string switchDp = fwNetMonitor_findSwitchDp(switchName);
  
  if(dpExists(switchDp))
  {  
    dpDelete(switchDp);
    //DebugTN("Deleted switch: " + switchDp);
  }
  else
  {
    //DebugTN(switchDp + " does not exist!");
    return FW_NET_MONITOR_ERROR_RETURN;
  }
  return 0;  
}

/** This function returns new DP address of the switch name 
    (but does not find corresponding DP if the switch name is 
    different than the name used to create DP)
  @param name (in) name of the nswitch (not DP address)
  @return DP address
*/
string fwNetMonitor_getSwitchDp(string name)
{
  strreplace(name, ":", "_");
  strreplace(name, ".", "_");
  strreplace(name, ",", "_");
  strreplace(name, " ", "_");
  strreplace(name, ";", "_");
  strreplace(name, "?", "_");
  strreplace(name, "!", "_");
  strreplace(name, "^", "_");
  strreplace(name, "@", "_");
  strreplace(name, ")", "_");
  strreplace(name, "(", "_");
  strreplace(name, "&", "_");
  strreplace(name, "%", "_");
  strreplace(name, "~", "_");
  
  return FW_NET_MONITOR_SWITCH_DP_PREFIX + name; 
}

/** This function finds the existing switch DP, basing on the name (searches through exisitng DPs)
    To be used especially when DP is not created from the current node name
    @param node (in) name of the switch (not DP address)
    @return DP address
*/
string fwNetMonitor_findSwitchDp(string switchName)
{
  dyn_string switchesDps;
  switchesDps = dpNames(FW_NET_MONITOR_SWITCH_DP_PREFIX+"*");
    
  string switchDp = "unknown DP";
  for(int i = 1; i <= dynlen(switchesDps); i++)
  {
    string name;
    dpGet(switchesDps[i]+".name",name);
    if(name == switchName)
    {
      switchDp = switchesDps[i];
      break;
    }
  }
  return switchDp;
}

/** This function sets switch "devices" DPE
  @param switchName (in) name of the switch (not DP address)
  @param devices (in) devices to be set
  @return 0
*/
int fwNetMonitor_setDevices(string switchName, dyn_string devices)
{
  string switchDp = fwNetMonitor_findSwitchDp(switchName);
  dpSetWait(switchDp + ".devices", devices);
  return 0;  
}

/** This function is callback from Control Manager script listening SPECTRUM
  @param dp (in) DataPoint of SPECTRUM message
  @param spectrumMessage (in) single SPECTRUM info
*/
void fwNetMonitor_analyseMessageCB(string dp, string spectrumMessage)
{
  dyn_string monitoredSwitches;
  dyn_string monitoredSwitchesDPs = dpNames(FW_NET_MONITOR_SWITCH_DP_PREFIX+"*.name", FW_NET_MONITOR_SWITCH_DP_TYPE);
  dpGet(monitoredSwitchesDPs, monitoredSwitches);
  
  /* 
    EXAMPLE SPECTRUM MESSAGE:
    SET 172.21.77.89 09/03/2013 06:17:38 T874-R-RHP82-1 6104402 1040b 0x2080093 "LINK DISABLEDSYMPTOMS:One or more of the ports involved in the link has been disabled.PROBABLE CAUSES:Administrator has disabled one or more ports involved in this link.RECOMMENDED ACTIONS:This status will change when the ports become enabled again."
  */

  //-----------------------------------
  // SPLIT THE MESSAGE AND ASSIGN TO THE INDIVIDUAL VARIABLES (sometimes we get an IP, sometimes we don't...)
  //----------------------------------- 
  
  // changes that were done: ds1 - messageDynAll;    ds2 - messageDynDetails;    ds3 - ipOrDate
  // code below is to analyse SPECTRUM message and retrieve info
  dyn_string messageDynAll = strsplit(spectrumMessage, "\"");
  dyn_string messageDynDetails = strsplit(messageDynAll[1], " ");
  dyn_string ipOrDate = strsplit(messageDynDetails[2], "."); // we don't know if message contains IP address
  
  string eventType, dateTime, switchName, description, ipAddress, alarmID, causeID;
  int state;  
  
  if(dynlen(ipOrDate) == 4) // ipAddress included
  {
    dateTime = messageDynDetails[3]+ " " + messageDynDetails[4];
    switchName = strtoupper(messageDynDetails[5]);
    alarmID = messageDynDetails[6];
    causeID = messageDynDetails[7];
    ipAddress = messageDynDetails[2];
  }
  else
  {
    dateTime = messageDynDetails[2]+ " " + messageDynDetails[3];
    switchName = strtoupper(messageDynDetails[4]);
    alarmID = messageDynDetails[5];
    causeID = messageDynDetails[6l];
  }
  
  int position = dynContains(monitoredSwitches, switchName);
  
  if(position > 0) // switch is monitored
  {
    eventType = messageDynDetails[1];
    description = messageDynAll[2];
  }
  //-----------------------------------
  // Multiinstance alert
  // Set additional values
  //----------------------------------- 
  
  time eventTime = fwNetMonitor_getTimeFromString(dateTime);
  string dpe = FW_NET_MONITOR_SWITCH_DP_PREFIX +switchName+".alarm";
  dyn_anytype addVal = makeDynAnytype(dateTime, eventType+": "+description, ipAddress, causeID, alarmID);
  
  
  //-----------------------------------
  // SET OR UPDATE AN ALARM
  //----------------------------------- 
  if( (eventType == "SET") || (eventType == "UPD") ) // Set alarm
  {
    dyn_errClass err;
    
    alertSet(eventTime, 0, dpe+":_alert_hdl.._event", DPATTR_ALERTEVENT_CAME, 
                       eventTime, 0, dpe+":_alert_hdl.._class", "_fwErrorNack_70.",
                       eventTime, 0, dpe+":_alert_hdl.._add_values", addVal,
                       eventTime, 0, dpe+":_alert_hdl.._alert_id", alarmID);
    err = getLastError();
   
    if(dynlen(err) > 0) 
    { 
      DebugN("An error has occurred:", err); 
    }
  }
  //-----------------------------------
  // OR DELETE AN ALARM
  //----------------------------------- 
  else if( eventType == "CLR" ) // clear alarm
  {   
    // ~~~~~~~~~~
    // QUERY assembly & call    
    // ~~~~~~~~~~
    
    // change: dda - queryResult
    dyn_dyn_anytype queryResult;
    string query = "SELECT ALERT '_alert_hdl.._class' FROM '"+dpe+"' WHERE '_alert_hdl.._alert_id' = \""+alarmID+"\"";
                  // AND '_alert_hdl.._came_time' = \"" + formatTime("%Y.%m.%d %H:%M:%S", t, ".%03d") + "\"";
    
    dpQuery(query, queryResult); 
    
    // ~~~~~~~~~~
    // RESULTS handling (clear the alert)    
    // ~~~~~~~~~~
    for(int i = 2; i <= dynlen(queryResult); i++)
    {
      int returnedInformation = alertSet((time)queryResult[i][2], getACount((atime)queryResult[i][2]), dpe+":_alert_hdl.._event", DPATTR_ALERTEVENT_WENT,
                         (time)queryResult[i][2], getACount((atime)queryResult[i][2]), dpe+":_alert_hdl.._add_values", addVal,
                         (time)queryResult[i][2], getACount((atime)queryResult[i][2]), dpe+":_alert_hdl.._alert_id", alarmID); 
      DebugTN( "Alert went.", returnedInformation, getLastError() ); 
    }
  }
  else
  {
    DebugTN( "fwNetMonitor_analyseMessageCB: ERROR - Unknown event type!!!" ); 
  }
  
} 

/** This function gets time variable from the string
  @param dateTime (in) string with time data in format: 09/03/2013 06:17:38
  @return time variable
*/
time fwNetMonitor_getTimeFromString(string dateTime)
{
  // time makeTime(unsigned year, unsigned month, unsigned day[, unsigned hour[, unsigned minute[, unsigned second[, unsigned milli[, bool daylightsaving]]]]]);
  
  dyn_string dateTimeDyn = strsplit(dateTime, " ");
  dyn_string dateDyn = strsplit(dateTimeDyn[1], "/");
  dyn_string timeDyn = strsplit(dateTimeDyn[2], ":");
  string tDay, tMonth, tYear, tHour, tMinute, tSecond; 
  tDay = dateDyn[1];
  tMonth = dateDyn[2];
  tYear = dateDyn[3];
  tHour = timeDyn[1];
  tMinute = timeDyn[2];
  tSecond = timeDyn[3];
  
  return makeTime(tYear, tMonth, tDay, tHour, tMinute, tSecond);
}

/** This function gets all the devices previously assigned to monitoring
  @param devices (out) list devices
  @return 0 if ok, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_getAllDevicesList(dyn_string &devices)
{
  dpGet("fwNetMonitorDevicesList.devices", devices);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) 
  { 
    DebugN("An error has occurred:", err);
    return FW_NET_MONITOR_ERROR_RETURN; 
  }
  return 0;  
}

/** This function sets all the devices to be monitored
  @param devices (in) list devices
  @return 0 if ok, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_setAllDevicesList(dyn_string devices)
{
  dpSetWait("fwNetMonitorDevicesList.devices", devices);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) 
  { 
    DebugN("An error has occurred:", err);
    return FW_NET_MONITOR_ERROR_RETURN; 
  }
  return 0;  
}

/** This function clears the devices list
  @return 0 if ok, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_clearAllDevicesList()
{
  dpSetWait("fwNetMonitorDevicesList.devices", "");
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) 
  { 
    DebugN("An error has occurred:", err);
    return FW_NET_MONITOR_ERROR_RETURN; 
  }
  return 0;  
}

/** This function appends new device to the monitored devices list
  @param device (in) debice name
  @return 0 if ok, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_appendDeviceToList(string newDevice)
{
  dyn_string devices;
  dpGet("fwNetMonitorDevicesList.devices", devices);
  dynAppend(devices, newDevice);
  dpSetWait("fwNetMonitorDevicesList.devices", devices);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) 
  { 
    DebugN("An error has occurred:", err);
    return FW_NET_MONITOR_ERROR_RETURN; 
  }
  return 0;  
}

/** This function removes device from the monitored devices list
  @param device (in) device name
  @return 0 if ok, FW_NET_MONITOR_ERROR_RETURN if error
*/
int fwNetMonitor_removeDeviceFromList(string deleteDevice)
{
  dyn_string devices;
  dpGet("fwNetMonitorDevicesList.devices", devices);
  int index = dynContains(devices, deleteDevice);
  if(index > 0)
  {
    dynRemove(devices,index);
    dpSetWait("fwNetMonitorDevicesList.devices", devices);
  }
  else
  {
    return FW_NET_MONITOR_ERROR_RETURN;
  }
  return 0;  
}

/** This function gets all the imported switches that contain specified devices
  @param devices (in) devices' names
  @return dynamic list of switches
*/
dyn_string fwNetMonitor_getDeviceSwitches(dyn_string devices)
{
  string switchName;
  dyn_string switches;
  dyn_string allSwitchDpes = dpNames(FW_NET_MONITOR_SWITCH_DP_PREFIX+"*", FW_NET_MONITOR_SWITCH_DP_TYPE); 
  //DebugTN("allSwitchDpes: ", allSwitchDpes);
  
  // iterate through switch dpe names
  for(int switchDpeIdx = 1; switchDpeIdx <= dynlen(allSwitchDpes); switchDpeIdx++)
  {
    dyn_string switchDevices;
    dpGet(allSwitchDpes[switchDpeIdx]+".devices",switchDevices);
    
    // for each switch we iterate through the switch's devices
    for(int deviceIdx = 1; deviceIdx <= dynlen(switchDevices); deviceIdx++)
    {
      // if the switch device name occurs in the list of devices we are interested in
      int devPositionInDevices = dynContains(devices, strtoupper(switchDevices[deviceIdx]));
      if(devPositionInDevices > 0)
      {
        // and it is not in the list of switches yet
        dpGet(allSwitchDpes[switchDpeIdx]+".name", switchName);
        if(dynContains(switches, switchName) == 0 )
        {   
          // then we append it to the list
          dynAppend(switches, switchName);
        }
      }
    }
  }
  return switches;
}
